// This file was generated by qtwaylandscanner
// source file is plasma-window-management.xml

#include "qwayland-plasma-window-management.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    org_kde_plasma_window_management::org_kde_plasma_window_management(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_plasma_window_management::org_kde_plasma_window_management(struct ::org_kde_plasma_window_management *obj)
        : m_org_kde_plasma_window_management(obj)
    {
        init_listener();
    }

    org_kde_plasma_window_management::org_kde_plasma_window_management()
        : m_org_kde_plasma_window_management(nullptr)
    {
    }

    org_kde_plasma_window_management::~org_kde_plasma_window_management()
    {
    }

    void org_kde_plasma_window_management::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_plasma_window_management = static_cast<struct ::org_kde_plasma_window_management *>(wlRegistryBind(registry, id, &org_kde_plasma_window_management_interface, version));
        init_listener();
    }

    void org_kde_plasma_window_management::init(struct ::org_kde_plasma_window_management *obj)
    {
        m_org_kde_plasma_window_management = obj;
        init_listener();
    }

    org_kde_plasma_window_management *org_kde_plasma_window_management::fromObject(struct ::org_kde_plasma_window_management *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_org_kde_plasma_window_management_listener)
            return nullptr;
        return static_cast<org_kde_plasma_window_management *>(org_kde_plasma_window_management_get_user_data(object));
    }

    bool org_kde_plasma_window_management::isInitialized() const
    {
        return m_org_kde_plasma_window_management != nullptr;
    }

    uint32_t org_kde_plasma_window_management::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_plasma_window_management));
    }

    const struct wl_interface *org_kde_plasma_window_management::interface()
    {
        return &::org_kde_plasma_window_management_interface;
    }

    void org_kde_plasma_window_management::show_desktop(uint32_t state)
    {
        ::org_kde_plasma_window_management_show_desktop(
            m_org_kde_plasma_window_management,
            state);
    }

    struct ::org_kde_plasma_window *org_kde_plasma_window_management::get_window(uint32_t internal_window_id)
    {
        struct ::org_kde_plasma_window * id = ::org_kde_plasma_window_management_get_window(
            m_org_kde_plasma_window_management,
            internal_window_id);
        return id;
    }

    struct ::org_kde_plasma_window *org_kde_plasma_window_management::get_window_by_uuid(const QString &internal_window_uuid)
    {
        struct ::org_kde_plasma_window * id = ::org_kde_plasma_window_management_get_window_by_uuid(
            m_org_kde_plasma_window_management,
            internal_window_uuid.toUtf8().constData());
        return id;
    }

    struct ::org_kde_plasma_stacking_order *org_kde_plasma_window_management::get_stacking_order()
    {
        struct ::org_kde_plasma_stacking_order * stacking_order = ::org_kde_plasma_window_management_get_stacking_order(
            m_org_kde_plasma_window_management);
        return stacking_order;
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_show_desktop_changed(uint32_t )
    {
    }

    void org_kde_plasma_window_management::handle_show_desktop_changed(
        void *data,
        struct ::org_kde_plasma_window_management *object,
        uint32_t state)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_show_desktop_changed(
            state);
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_window(uint32_t )
    {
    }

    void org_kde_plasma_window_management::handle_window(
        void *data,
        struct ::org_kde_plasma_window_management *object,
        uint32_t id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_window(
            id);
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_stacking_order_changed(wl_array *)
    {
    }

    void org_kde_plasma_window_management::handle_stacking_order_changed(
        void *data,
        struct ::org_kde_plasma_window_management *object,
        wl_array *ids)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_stacking_order_changed(
            ids);
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_stacking_order_uuid_changed(const QString &)
    {
    }

    void org_kde_plasma_window_management::handle_stacking_order_uuid_changed(
        void *data,
        struct ::org_kde_plasma_window_management *object,
        const char *uuids)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_stacking_order_uuid_changed(
            QString::fromUtf8(uuids));
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_window_with_uuid(uint32_t , const QString &)
    {
    }

    void org_kde_plasma_window_management::handle_window_with_uuid(
        void *data,
        struct ::org_kde_plasma_window_management *object,
        uint32_t id,
        const char *uuid)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_window_with_uuid(
            id,
            QString::fromUtf8(uuid));
    }

    void org_kde_plasma_window_management::org_kde_plasma_window_management_stacking_order_changed_2()
    {
    }

    void org_kde_plasma_window_management::handle_stacking_order_changed_2(
        void *data,
        struct ::org_kde_plasma_window_management *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window_management *>(data)->org_kde_plasma_window_management_stacking_order_changed_2();
    }

    const struct org_kde_plasma_window_management_listener org_kde_plasma_window_management::m_org_kde_plasma_window_management_listener = {
        org_kde_plasma_window_management::handle_show_desktop_changed,
        org_kde_plasma_window_management::handle_window,
        org_kde_plasma_window_management::handle_stacking_order_changed,
        org_kde_plasma_window_management::handle_stacking_order_uuid_changed,
        org_kde_plasma_window_management::handle_window_with_uuid,
        org_kde_plasma_window_management::handle_stacking_order_changed_2,
    };

    void org_kde_plasma_window_management::init_listener()
    {
        org_kde_plasma_window_management_add_listener(m_org_kde_plasma_window_management, &m_org_kde_plasma_window_management_listener, this);
    }

    org_kde_plasma_window::org_kde_plasma_window(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_plasma_window::org_kde_plasma_window(struct ::org_kde_plasma_window *obj)
        : m_org_kde_plasma_window(obj)
    {
        init_listener();
    }

    org_kde_plasma_window::org_kde_plasma_window()
        : m_org_kde_plasma_window(nullptr)
    {
    }

    org_kde_plasma_window::~org_kde_plasma_window()
    {
    }

    void org_kde_plasma_window::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_plasma_window = static_cast<struct ::org_kde_plasma_window *>(wlRegistryBind(registry, id, &org_kde_plasma_window_interface, version));
        init_listener();
    }

    void org_kde_plasma_window::init(struct ::org_kde_plasma_window *obj)
    {
        m_org_kde_plasma_window = obj;
        init_listener();
    }

    org_kde_plasma_window *org_kde_plasma_window::fromObject(struct ::org_kde_plasma_window *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_org_kde_plasma_window_listener)
            return nullptr;
        return static_cast<org_kde_plasma_window *>(org_kde_plasma_window_get_user_data(object));
    }

    bool org_kde_plasma_window::isInitialized() const
    {
        return m_org_kde_plasma_window != nullptr;
    }

    uint32_t org_kde_plasma_window::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_plasma_window));
    }

    const struct wl_interface *org_kde_plasma_window::interface()
    {
        return &::org_kde_plasma_window_interface;
    }

    void org_kde_plasma_window::set_state(uint32_t flags, uint32_t state)
    {
        ::org_kde_plasma_window_set_state(
            m_org_kde_plasma_window,
            flags,
            state);
    }

    void org_kde_plasma_window::set_virtual_desktop(uint32_t number)
    {
        ::org_kde_plasma_window_set_virtual_desktop(
            m_org_kde_plasma_window,
            number);
    }

    void org_kde_plasma_window::set_minimized_geometry(struct ::wl_surface *panel, uint32_t x, uint32_t y, uint32_t width, uint32_t height)
    {
        ::org_kde_plasma_window_set_minimized_geometry(
            m_org_kde_plasma_window,
            panel,
            x,
            y,
            width,
            height);
    }

    void org_kde_plasma_window::unset_minimized_geometry(struct ::wl_surface *panel)
    {
        ::org_kde_plasma_window_unset_minimized_geometry(
            m_org_kde_plasma_window,
            panel);
    }

    void org_kde_plasma_window::close()
    {
        ::org_kde_plasma_window_close(
            m_org_kde_plasma_window);
    }

    void org_kde_plasma_window::request_move()
    {
        ::org_kde_plasma_window_request_move(
            m_org_kde_plasma_window);
    }

    void org_kde_plasma_window::request_resize()
    {
        ::org_kde_plasma_window_request_resize(
            m_org_kde_plasma_window);
    }

    void org_kde_plasma_window::destroy()
    {
        ::org_kde_plasma_window_destroy(
            m_org_kde_plasma_window);
        m_org_kde_plasma_window = nullptr;
    }

    void org_kde_plasma_window::get_icon(int32_t fd)
    {
        ::org_kde_plasma_window_get_icon(
            m_org_kde_plasma_window,
            fd);
    }

    void org_kde_plasma_window::request_enter_virtual_desktop(const QString &id)
    {
        ::org_kde_plasma_window_request_enter_virtual_desktop(
            m_org_kde_plasma_window,
            id.toUtf8().constData());
    }

    void org_kde_plasma_window::request_enter_new_virtual_desktop()
    {
        ::org_kde_plasma_window_request_enter_new_virtual_desktop(
            m_org_kde_plasma_window);
    }

    void org_kde_plasma_window::request_leave_virtual_desktop(const QString &id)
    {
        ::org_kde_plasma_window_request_leave_virtual_desktop(
            m_org_kde_plasma_window,
            id.toUtf8().constData());
    }

    void org_kde_plasma_window::request_enter_activity(const QString &id)
    {
        ::org_kde_plasma_window_request_enter_activity(
            m_org_kde_plasma_window,
            id.toUtf8().constData());
    }

    void org_kde_plasma_window::request_leave_activity(const QString &id)
    {
        ::org_kde_plasma_window_request_leave_activity(
            m_org_kde_plasma_window,
            id.toUtf8().constData());
    }

    void org_kde_plasma_window::send_to_output(struct ::wl_output *output)
    {
        ::org_kde_plasma_window_send_to_output(
            m_org_kde_plasma_window,
            output);
    }

    void org_kde_plasma_window::org_kde_plasma_window_title_changed(const QString &)
    {
    }

    void org_kde_plasma_window::handle_title_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *title)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_title_changed(
            QString::fromUtf8(title));
    }

    void org_kde_plasma_window::org_kde_plasma_window_app_id_changed(const QString &)
    {
    }

    void org_kde_plasma_window::handle_app_id_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *app_id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_app_id_changed(
            QString::fromUtf8(app_id));
    }

    void org_kde_plasma_window::org_kde_plasma_window_state_changed(uint32_t )
    {
    }

    void org_kde_plasma_window::handle_state_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        uint32_t flags)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_state_changed(
            flags);
    }

    void org_kde_plasma_window::org_kde_plasma_window_virtual_desktop_changed(int32_t )
    {
    }

    void org_kde_plasma_window::handle_virtual_desktop_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        int32_t number)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_virtual_desktop_changed(
            number);
    }

    void org_kde_plasma_window::org_kde_plasma_window_themed_icon_name_changed(const QString &)
    {
    }

    void org_kde_plasma_window::handle_themed_icon_name_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *name)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_themed_icon_name_changed(
            QString::fromUtf8(name));
    }

    void org_kde_plasma_window::org_kde_plasma_window_unmapped()
    {
    }

    void org_kde_plasma_window::handle_unmapped(
        void *data,
        struct ::org_kde_plasma_window *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_unmapped();
    }

    void org_kde_plasma_window::org_kde_plasma_window_initial_state()
    {
    }

    void org_kde_plasma_window::handle_initial_state(
        void *data,
        struct ::org_kde_plasma_window *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_initial_state();
    }

    void org_kde_plasma_window::org_kde_plasma_window_parent_window(struct ::org_kde_plasma_window *)
    {
    }

    void org_kde_plasma_window::handle_parent_window(
        void *data,
        struct ::org_kde_plasma_window *object,
        struct ::org_kde_plasma_window *parent)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_parent_window(
            parent);
    }

    void org_kde_plasma_window::org_kde_plasma_window_geometry(int32_t , int32_t , uint32_t , uint32_t )
    {
    }

    void org_kde_plasma_window::handle_geometry(
        void *data,
        struct ::org_kde_plasma_window *object,
        int32_t x,
        int32_t y,
        uint32_t width,
        uint32_t height)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_geometry(
            x,
            y,
            width,
            height);
    }

    void org_kde_plasma_window::org_kde_plasma_window_icon_changed()
    {
    }

    void org_kde_plasma_window::handle_icon_changed(
        void *data,
        struct ::org_kde_plasma_window *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_icon_changed();
    }

    void org_kde_plasma_window::org_kde_plasma_window_pid_changed(uint32_t )
    {
    }

    void org_kde_plasma_window::handle_pid_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        uint32_t pid)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_pid_changed(
            pid);
    }

    void org_kde_plasma_window::org_kde_plasma_window_virtual_desktop_entered(const QString &)
    {
    }

    void org_kde_plasma_window::handle_virtual_desktop_entered(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_virtual_desktop_entered(
            QString::fromUtf8(id));
    }

    void org_kde_plasma_window::org_kde_plasma_window_virtual_desktop_left(const QString &)
    {
    }

    void org_kde_plasma_window::handle_virtual_desktop_left(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *is)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_virtual_desktop_left(
            QString::fromUtf8(is));
    }

    void org_kde_plasma_window::org_kde_plasma_window_application_menu(const QString &, const QString &)
    {
    }

    void org_kde_plasma_window::handle_application_menu(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *service_name,
        const char *object_path)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_application_menu(
            QString::fromUtf8(service_name),
            QString::fromUtf8(object_path));
    }

    void org_kde_plasma_window::org_kde_plasma_window_activity_entered(const QString &)
    {
    }

    void org_kde_plasma_window::handle_activity_entered(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_activity_entered(
            QString::fromUtf8(id));
    }

    void org_kde_plasma_window::org_kde_plasma_window_activity_left(const QString &)
    {
    }

    void org_kde_plasma_window::handle_activity_left(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_activity_left(
            QString::fromUtf8(id));
    }

    void org_kde_plasma_window::org_kde_plasma_window_resource_name_changed(const QString &)
    {
    }

    void org_kde_plasma_window::handle_resource_name_changed(
        void *data,
        struct ::org_kde_plasma_window *object,
        const char *resource_name)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_resource_name_changed(
            QString::fromUtf8(resource_name));
    }

    void org_kde_plasma_window::org_kde_plasma_window_client_geometry(int32_t , int32_t , uint32_t , uint32_t )
    {
    }

    void org_kde_plasma_window::handle_client_geometry(
        void *data,
        struct ::org_kde_plasma_window *object,
        int32_t x,
        int32_t y,
        uint32_t width,
        uint32_t height)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_window *>(data)->org_kde_plasma_window_client_geometry(
            x,
            y,
            width,
            height);
    }

    const struct org_kde_plasma_window_listener org_kde_plasma_window::m_org_kde_plasma_window_listener = {
        org_kde_plasma_window::handle_title_changed,
        org_kde_plasma_window::handle_app_id_changed,
        org_kde_plasma_window::handle_state_changed,
        org_kde_plasma_window::handle_virtual_desktop_changed,
        org_kde_plasma_window::handle_themed_icon_name_changed,
        org_kde_plasma_window::handle_unmapped,
        org_kde_plasma_window::handle_initial_state,
        org_kde_plasma_window::handle_parent_window,
        org_kde_plasma_window::handle_geometry,
        org_kde_plasma_window::handle_icon_changed,
        org_kde_plasma_window::handle_pid_changed,
        org_kde_plasma_window::handle_virtual_desktop_entered,
        org_kde_plasma_window::handle_virtual_desktop_left,
        org_kde_plasma_window::handle_application_menu,
        org_kde_plasma_window::handle_activity_entered,
        org_kde_plasma_window::handle_activity_left,
        org_kde_plasma_window::handle_resource_name_changed,
        org_kde_plasma_window::handle_client_geometry,
    };

    void org_kde_plasma_window::init_listener()
    {
        org_kde_plasma_window_add_listener(m_org_kde_plasma_window, &m_org_kde_plasma_window_listener, this);
    }

    org_kde_plasma_activation_feedback::org_kde_plasma_activation_feedback(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_plasma_activation_feedback::org_kde_plasma_activation_feedback(struct ::org_kde_plasma_activation_feedback *obj)
        : m_org_kde_plasma_activation_feedback(obj)
    {
        init_listener();
    }

    org_kde_plasma_activation_feedback::org_kde_plasma_activation_feedback()
        : m_org_kde_plasma_activation_feedback(nullptr)
    {
    }

    org_kde_plasma_activation_feedback::~org_kde_plasma_activation_feedback()
    {
    }

    void org_kde_plasma_activation_feedback::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_plasma_activation_feedback = static_cast<struct ::org_kde_plasma_activation_feedback *>(wlRegistryBind(registry, id, &org_kde_plasma_activation_feedback_interface, version));
        init_listener();
    }

    void org_kde_plasma_activation_feedback::init(struct ::org_kde_plasma_activation_feedback *obj)
    {
        m_org_kde_plasma_activation_feedback = obj;
        init_listener();
    }

    org_kde_plasma_activation_feedback *org_kde_plasma_activation_feedback::fromObject(struct ::org_kde_plasma_activation_feedback *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_org_kde_plasma_activation_feedback_listener)
            return nullptr;
        return static_cast<org_kde_plasma_activation_feedback *>(org_kde_plasma_activation_feedback_get_user_data(object));
    }

    bool org_kde_plasma_activation_feedback::isInitialized() const
    {
        return m_org_kde_plasma_activation_feedback != nullptr;
    }

    uint32_t org_kde_plasma_activation_feedback::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_plasma_activation_feedback));
    }

    const struct wl_interface *org_kde_plasma_activation_feedback::interface()
    {
        return &::org_kde_plasma_activation_feedback_interface;
    }

    void org_kde_plasma_activation_feedback::destroy()
    {
        ::org_kde_plasma_activation_feedback_destroy(
            m_org_kde_plasma_activation_feedback);
        m_org_kde_plasma_activation_feedback = nullptr;
    }

    void org_kde_plasma_activation_feedback::org_kde_plasma_activation_feedback_activation(struct ::org_kde_plasma_activation *)
    {
    }

    void org_kde_plasma_activation_feedback::handle_activation(
        void *data,
        struct ::org_kde_plasma_activation_feedback *object,
        struct ::org_kde_plasma_activation *id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_activation_feedback *>(data)->org_kde_plasma_activation_feedback_activation(
            id);
    }

    const struct org_kde_plasma_activation_feedback_listener org_kde_plasma_activation_feedback::m_org_kde_plasma_activation_feedback_listener = {
        org_kde_plasma_activation_feedback::handle_activation,
    };

    void org_kde_plasma_activation_feedback::init_listener()
    {
        org_kde_plasma_activation_feedback_add_listener(m_org_kde_plasma_activation_feedback, &m_org_kde_plasma_activation_feedback_listener, this);
    }

    org_kde_plasma_activation::org_kde_plasma_activation(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_plasma_activation::org_kde_plasma_activation(struct ::org_kde_plasma_activation *obj)
        : m_org_kde_plasma_activation(obj)
    {
        init_listener();
    }

    org_kde_plasma_activation::org_kde_plasma_activation()
        : m_org_kde_plasma_activation(nullptr)
    {
    }

    org_kde_plasma_activation::~org_kde_plasma_activation()
    {
    }

    void org_kde_plasma_activation::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_plasma_activation = static_cast<struct ::org_kde_plasma_activation *>(wlRegistryBind(registry, id, &org_kde_plasma_activation_interface, version));
        init_listener();
    }

    void org_kde_plasma_activation::init(struct ::org_kde_plasma_activation *obj)
    {
        m_org_kde_plasma_activation = obj;
        init_listener();
    }

    org_kde_plasma_activation *org_kde_plasma_activation::fromObject(struct ::org_kde_plasma_activation *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_org_kde_plasma_activation_listener)
            return nullptr;
        return static_cast<org_kde_plasma_activation *>(org_kde_plasma_activation_get_user_data(object));
    }

    bool org_kde_plasma_activation::isInitialized() const
    {
        return m_org_kde_plasma_activation != nullptr;
    }

    uint32_t org_kde_plasma_activation::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_plasma_activation));
    }

    const struct wl_interface *org_kde_plasma_activation::interface()
    {
        return &::org_kde_plasma_activation_interface;
    }

    void org_kde_plasma_activation::destroy()
    {
        ::org_kde_plasma_activation_destroy(
            m_org_kde_plasma_activation);
        m_org_kde_plasma_activation = nullptr;
    }

    void org_kde_plasma_activation::org_kde_plasma_activation_app_id(const QString &)
    {
    }

    void org_kde_plasma_activation::handle_app_id(
        void *data,
        struct ::org_kde_plasma_activation *object,
        const char *app_id)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_activation *>(data)->org_kde_plasma_activation_app_id(
            QString::fromUtf8(app_id));
    }

    void org_kde_plasma_activation::org_kde_plasma_activation_finished()
    {
    }

    void org_kde_plasma_activation::handle_finished(
        void *data,
        struct ::org_kde_plasma_activation *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_activation *>(data)->org_kde_plasma_activation_finished();
    }

    const struct org_kde_plasma_activation_listener org_kde_plasma_activation::m_org_kde_plasma_activation_listener = {
        org_kde_plasma_activation::handle_app_id,
        org_kde_plasma_activation::handle_finished,
    };

    void org_kde_plasma_activation::init_listener()
    {
        org_kde_plasma_activation_add_listener(m_org_kde_plasma_activation, &m_org_kde_plasma_activation_listener, this);
    }

    org_kde_plasma_stacking_order::org_kde_plasma_stacking_order(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_plasma_stacking_order::org_kde_plasma_stacking_order(struct ::org_kde_plasma_stacking_order *obj)
        : m_org_kde_plasma_stacking_order(obj)
    {
        init_listener();
    }

    org_kde_plasma_stacking_order::org_kde_plasma_stacking_order()
        : m_org_kde_plasma_stacking_order(nullptr)
    {
    }

    org_kde_plasma_stacking_order::~org_kde_plasma_stacking_order()
    {
    }

    void org_kde_plasma_stacking_order::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_plasma_stacking_order = static_cast<struct ::org_kde_plasma_stacking_order *>(wlRegistryBind(registry, id, &org_kde_plasma_stacking_order_interface, version));
        init_listener();
    }

    void org_kde_plasma_stacking_order::init(struct ::org_kde_plasma_stacking_order *obj)
    {
        m_org_kde_plasma_stacking_order = obj;
        init_listener();
    }

    org_kde_plasma_stacking_order *org_kde_plasma_stacking_order::fromObject(struct ::org_kde_plasma_stacking_order *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_org_kde_plasma_stacking_order_listener)
            return nullptr;
        return static_cast<org_kde_plasma_stacking_order *>(org_kde_plasma_stacking_order_get_user_data(object));
    }

    bool org_kde_plasma_stacking_order::isInitialized() const
    {
        return m_org_kde_plasma_stacking_order != nullptr;
    }

    uint32_t org_kde_plasma_stacking_order::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_plasma_stacking_order));
    }

    const struct wl_interface *org_kde_plasma_stacking_order::interface()
    {
        return &::org_kde_plasma_stacking_order_interface;
    }

    void org_kde_plasma_stacking_order::org_kde_plasma_stacking_order_window(const QString &)
    {
    }

    void org_kde_plasma_stacking_order::handle_window(
        void *data,
        struct ::org_kde_plasma_stacking_order *object,
        const char *uuid)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_stacking_order *>(data)->org_kde_plasma_stacking_order_window(
            QString::fromUtf8(uuid));
    }

    void org_kde_plasma_stacking_order::org_kde_plasma_stacking_order_done()
    {
    }

    void org_kde_plasma_stacking_order::handle_done(
        void *data,
        struct ::org_kde_plasma_stacking_order *object)
    {
        Q_UNUSED(object);
        static_cast<org_kde_plasma_stacking_order *>(data)->org_kde_plasma_stacking_order_done();
    }

    const struct org_kde_plasma_stacking_order_listener org_kde_plasma_stacking_order::m_org_kde_plasma_stacking_order_listener = {
        org_kde_plasma_stacking_order::handle_window,
        org_kde_plasma_stacking_order::handle_done,
    };

    void org_kde_plasma_stacking_order::init_listener()
    {
        org_kde_plasma_stacking_order_add_listener(m_org_kde_plasma_stacking_order, &m_org_kde_plasma_stacking_order_listener, this);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
