// This file was generated by qtwaylandscanner
// source file is xdg-dialog-v1.xml

#ifndef QT_WAYLAND_XDG_DIALOG_V1
#define QT_WAYLAND_XDG_DIALOG_V1

#include "wayland-xdg-dialog-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  xdg_wm_dialog_v1
    {
    public:
        xdg_wm_dialog_v1(struct ::wl_registry *registry, uint32_t id, int version);
        xdg_wm_dialog_v1(struct ::xdg_wm_dialog_v1 *object);
        xdg_wm_dialog_v1();

        virtual ~xdg_wm_dialog_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::xdg_wm_dialog_v1 *object);

        struct ::xdg_wm_dialog_v1 *object() { return m_xdg_wm_dialog_v1; }
        const struct ::xdg_wm_dialog_v1 *object() const { return m_xdg_wm_dialog_v1; }
        static xdg_wm_dialog_v1 *fromObject(struct ::xdg_wm_dialog_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_already_used = 0, // the xdg_toplevel object has already been used to create a xdg_dialog_v1
        };

        void destroy();
        struct ::xdg_dialog_v1 *get_xdg_dialog(struct ::xdg_toplevel *toplevel);

    private:
        struct ::xdg_wm_dialog_v1 *m_xdg_wm_dialog_v1;
    };

    class  xdg_dialog_v1
    {
    public:
        xdg_dialog_v1(struct ::wl_registry *registry, uint32_t id, int version);
        xdg_dialog_v1(struct ::xdg_dialog_v1 *object);
        xdg_dialog_v1();

        virtual ~xdg_dialog_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::xdg_dialog_v1 *object);

        struct ::xdg_dialog_v1 *object() { return m_xdg_dialog_v1; }
        const struct ::xdg_dialog_v1 *object() const { return m_xdg_dialog_v1; }
        static xdg_dialog_v1 *fromObject(struct ::xdg_dialog_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        void set_modal();
        void unset_modal();

    private:
        struct ::xdg_dialog_v1 *m_xdg_dialog_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
