// This file was generated by qtwaylandscanner
// source file is xdg-toplevel-tag-v1.xml

#ifndef QT_WAYLAND_XDG_TOPLEVEL_TAG_V1
#define QT_WAYLAND_XDG_TOPLEVEL_TAG_V1

#include "wayland-xdg-toplevel-tag-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  xdg_toplevel_tag_manager_v1
    {
    public:
        xdg_toplevel_tag_manager_v1(struct ::wl_registry *registry, uint32_t id, int version);
        xdg_toplevel_tag_manager_v1(struct ::xdg_toplevel_tag_manager_v1 *object);
        xdg_toplevel_tag_manager_v1();

        virtual ~xdg_toplevel_tag_manager_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::xdg_toplevel_tag_manager_v1 *object);

        struct ::xdg_toplevel_tag_manager_v1 *object() { return m_xdg_toplevel_tag_manager_v1; }
        const struct ::xdg_toplevel_tag_manager_v1 *object() const { return m_xdg_toplevel_tag_manager_v1; }
        static xdg_toplevel_tag_manager_v1 *fromObject(struct ::xdg_toplevel_tag_manager_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        void set_toplevel_tag(struct ::xdg_toplevel *toplevel, const QString &tag);
        void set_toplevel_description(struct ::xdg_toplevel *toplevel, const QString &description);

    private:
        struct ::xdg_toplevel_tag_manager_v1 *m_xdg_toplevel_tag_manager_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
