// This file was generated by qtwaylandscanner
// source file is contrast.xml

#ifndef QT_WAYLAND_CONTRAST
#define QT_WAYLAND_CONTRAST

#include "wayland-contrast-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  org_kde_kwin_contrast_manager
    {
    public:
        org_kde_kwin_contrast_manager(struct ::wl_registry *registry, uint32_t id, int version);
        org_kde_kwin_contrast_manager(struct ::org_kde_kwin_contrast_manager *object);
        org_kde_kwin_contrast_manager();

        virtual ~org_kde_kwin_contrast_manager();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::org_kde_kwin_contrast_manager *object);

        struct ::org_kde_kwin_contrast_manager *object() { return m_org_kde_kwin_contrast_manager; }
        const struct ::org_kde_kwin_contrast_manager *object() const { return m_org_kde_kwin_contrast_manager; }
        static org_kde_kwin_contrast_manager *fromObject(struct ::org_kde_kwin_contrast_manager *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        struct ::org_kde_kwin_contrast *create(struct ::wl_surface *surface);
        void unset(struct ::wl_surface *surface);

    private:
        struct ::org_kde_kwin_contrast_manager *m_org_kde_kwin_contrast_manager;
    };

    class  org_kde_kwin_contrast
    {
    public:
        org_kde_kwin_contrast(struct ::wl_registry *registry, uint32_t id, int version);
        org_kde_kwin_contrast(struct ::org_kde_kwin_contrast *object);
        org_kde_kwin_contrast();

        virtual ~org_kde_kwin_contrast();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::org_kde_kwin_contrast *object);

        struct ::org_kde_kwin_contrast *object() { return m_org_kde_kwin_contrast; }
        const struct ::org_kde_kwin_contrast *object() const { return m_org_kde_kwin_contrast; }
        static org_kde_kwin_contrast *fromObject(struct ::org_kde_kwin_contrast *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void commit();
        void set_region(struct ::wl_region *region);
        void set_contrast(wl_fixed_t contrast);
        void set_intensity(wl_fixed_t intensity);
        void set_saturation(wl_fixed_t saturation);
        void release();
        void set_frost(int32_t red, int32_t green, int32_t blue, int32_t alpha);
        void unset_frost();

    private:
        struct ::org_kde_kwin_contrast *m_org_kde_kwin_contrast;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
