// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kwindowsystem_xcb_debug.h"


Q_LOGGING_CATEGORY(LOG_KKEYSERVER_X11, "kf.windowsystem.keyserver.x11", QtInfoMsg)

