/****************************************************************************
** Meta object code from reading C++ file 'kx11extras.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kwindowsystem-6.29.0/src/kx11extras.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kx11extras.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KX11ExtrasE_t {};
} // unnamed namespace

template <> constexpr inline auto KX11Extras::qt_create_metaobjectdata<qt_meta_tag_ZN10KX11ExtrasE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KX11Extras",
        "currentDesktopChanged",
        "",
        "desktop",
        "windowAdded",
        "WId",
        "id",
        "windowRemoved",
        "activeWindowChanged",
        "desktopNamesChanged",
        "numberOfDesktopsChanged",
        "num",
        "workAreaChanged",
        "strutChanged",
        "stackingOrderChanged",
        "windowChanged",
        "NET::Properties",
        "properties",
        "NET::Properties2",
        "properties2",
        "compositingChanged",
        "enabled",
        "forceActiveWindow",
        "QWindow*",
        "window",
        "time",
        "compositingActive"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentDesktopChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'windowAdded'
        QtMocHelpers::SignalData<void(WId)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'windowRemoved'
        QtMocHelpers::SignalData<void(WId)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'activeWindowChanged'
        QtMocHelpers::SignalData<void(WId)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'desktopNamesChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'numberOfDesktopsChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'workAreaChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'strutChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stackingOrderChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowChanged'
        QtMocHelpers::SignalData<void(WId, NET::Properties, NET::Properties2)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Signal 'compositingChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Method 'forceActiveWindow'
        QtMocHelpers::MethodData<void(QWindow *, long)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::Long, 25 },
        }}),
        // Method 'forceActiveWindow'
        QtMocHelpers::MethodData<void(QWindow *)>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'compositingActive'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KX11Extras, qt_meta_tag_ZN10KX11ExtrasE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KX11Extras::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KX11ExtrasE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KX11ExtrasE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KX11ExtrasE_t>.metaTypes,
    nullptr
} };

void KX11Extras::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KX11Extras *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentDesktopChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->windowAdded((*reinterpret_cast<std::add_pointer_t<WId>>(_a[1]))); break;
        case 2: _t->windowRemoved((*reinterpret_cast<std::add_pointer_t<WId>>(_a[1]))); break;
        case 3: _t->activeWindowChanged((*reinterpret_cast<std::add_pointer_t<WId>>(_a[1]))); break;
        case 4: _t->desktopNamesChanged(); break;
        case 5: _t->numberOfDesktopsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->workAreaChanged(); break;
        case 7: _t->strutChanged(); break;
        case 8: _t->stackingOrderChanged(); break;
        case 9: _t->windowChanged((*reinterpret_cast<std::add_pointer_t<WId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NET::Properties>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<NET::Properties2>>(_a[3]))); break;
        case 10: _t->compositingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->forceActiveWindow((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<long>>(_a[2]))); break;
        case 12: _t->forceActiveWindow((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWindow* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWindow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(int )>(_a, &KX11Extras::currentDesktopChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(WId )>(_a, &KX11Extras::windowAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(WId )>(_a, &KX11Extras::windowRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(WId )>(_a, &KX11Extras::activeWindowChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)()>(_a, &KX11Extras::desktopNamesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(int )>(_a, &KX11Extras::numberOfDesktopsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)()>(_a, &KX11Extras::workAreaChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)()>(_a, &KX11Extras::strutChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)()>(_a, &KX11Extras::stackingOrderChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(WId , NET::Properties , NET::Properties2 )>(_a, &KX11Extras::windowChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KX11Extras::*)(bool )>(_a, &KX11Extras::compositingChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->compositingActive(); break;
        default: break;
        }
    }
}

const QMetaObject *KX11Extras::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KX11Extras::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KX11ExtrasE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KX11Extras::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KX11Extras::currentDesktopChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KX11Extras::windowAdded(WId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KX11Extras::windowRemoved(WId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KX11Extras::activeWindowChanged(WId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KX11Extras::desktopNamesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KX11Extras::numberOfDesktopsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KX11Extras::workAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KX11Extras::strutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KX11Extras::stackingOrderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KX11Extras::windowChanged(WId _t1, NET::Properties _t2, NET::Properties2 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void KX11Extras::compositingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
