// This file was generated by qtwaylandscanner
// source file is shadow.xml

#ifndef QT_WAYLAND_SHADOW
#define QT_WAYLAND_SHADOW

#include "wayland-shadow-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  org_kde_kwin_shadow_manager
    {
    public:
        org_kde_kwin_shadow_manager(struct ::wl_registry *registry, uint32_t id, int version);
        org_kde_kwin_shadow_manager(struct ::org_kde_kwin_shadow_manager *object);
        org_kde_kwin_shadow_manager();

        virtual ~org_kde_kwin_shadow_manager();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::org_kde_kwin_shadow_manager *object);

        struct ::org_kde_kwin_shadow_manager *object() { return m_org_kde_kwin_shadow_manager; }
        const struct ::org_kde_kwin_shadow_manager *object() const { return m_org_kde_kwin_shadow_manager; }
        static org_kde_kwin_shadow_manager *fromObject(struct ::org_kde_kwin_shadow_manager *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        struct ::org_kde_kwin_shadow *create(struct ::wl_surface *surface);
        void unset(struct ::wl_surface *surface);
        void destroy();

    private:
        struct ::org_kde_kwin_shadow_manager *m_org_kde_kwin_shadow_manager;
    };

    class  org_kde_kwin_shadow
    {
    public:
        org_kde_kwin_shadow(struct ::wl_registry *registry, uint32_t id, int version);
        org_kde_kwin_shadow(struct ::org_kde_kwin_shadow *object);
        org_kde_kwin_shadow();

        virtual ~org_kde_kwin_shadow();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::org_kde_kwin_shadow *object);

        struct ::org_kde_kwin_shadow *object() { return m_org_kde_kwin_shadow; }
        const struct ::org_kde_kwin_shadow *object() const { return m_org_kde_kwin_shadow; }
        static org_kde_kwin_shadow *fromObject(struct ::org_kde_kwin_shadow *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void commit();
        void attach_left(struct ::wl_buffer *buffer);
        void attach_top_left(struct ::wl_buffer *buffer);
        void attach_top(struct ::wl_buffer *buffer);
        void attach_top_right(struct ::wl_buffer *buffer);
        void attach_right(struct ::wl_buffer *buffer);
        void attach_bottom_right(struct ::wl_buffer *buffer);
        void attach_bottom(struct ::wl_buffer *buffer);
        void attach_bottom_left(struct ::wl_buffer *buffer);
        void set_left_offset(wl_fixed_t offset);
        void set_top_offset(wl_fixed_t offset);
        void set_right_offset(wl_fixed_t offset);
        void set_bottom_offset(wl_fixed_t offset);
        void destroy();

    private:
        struct ::org_kde_kwin_shadow *m_org_kde_kwin_shadow;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
