// This file was generated by qtwaylandscanner
// source file is shadow.xml

#include "qwayland-shadow.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    org_kde_kwin_shadow_manager::org_kde_kwin_shadow_manager(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_kwin_shadow_manager::org_kde_kwin_shadow_manager(struct ::org_kde_kwin_shadow_manager *obj)
        : m_org_kde_kwin_shadow_manager(obj)
    {
    }

    org_kde_kwin_shadow_manager::org_kde_kwin_shadow_manager()
        : m_org_kde_kwin_shadow_manager(nullptr)
    {
    }

    org_kde_kwin_shadow_manager::~org_kde_kwin_shadow_manager()
    {
    }

    void org_kde_kwin_shadow_manager::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_kwin_shadow_manager = static_cast<struct ::org_kde_kwin_shadow_manager *>(wlRegistryBind(registry, id, &org_kde_kwin_shadow_manager_interface, version));
    }

    void org_kde_kwin_shadow_manager::init(struct ::org_kde_kwin_shadow_manager *obj)
    {
        m_org_kde_kwin_shadow_manager = obj;
    }

    org_kde_kwin_shadow_manager *org_kde_kwin_shadow_manager::fromObject(struct ::org_kde_kwin_shadow_manager *object)
    {
        return static_cast<org_kde_kwin_shadow_manager *>(org_kde_kwin_shadow_manager_get_user_data(object));
    }

    bool org_kde_kwin_shadow_manager::isInitialized() const
    {
        return m_org_kde_kwin_shadow_manager != nullptr;
    }

    uint32_t org_kde_kwin_shadow_manager::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_kwin_shadow_manager));
    }

    const struct wl_interface *org_kde_kwin_shadow_manager::interface()
    {
        return &::org_kde_kwin_shadow_manager_interface;
    }

    struct ::org_kde_kwin_shadow *org_kde_kwin_shadow_manager::create(struct ::wl_surface *surface)
    {
        struct ::org_kde_kwin_shadow * id = ::org_kde_kwin_shadow_manager_create(
            m_org_kde_kwin_shadow_manager,
            surface);
        return id;
    }

    void org_kde_kwin_shadow_manager::unset(struct ::wl_surface *surface)
    {
        ::org_kde_kwin_shadow_manager_unset(
            m_org_kde_kwin_shadow_manager,
            surface);
    }

    void org_kde_kwin_shadow_manager::destroy()
    {
        ::org_kde_kwin_shadow_manager_destroy(
            m_org_kde_kwin_shadow_manager);
        m_org_kde_kwin_shadow_manager = nullptr;
    }

    org_kde_kwin_shadow::org_kde_kwin_shadow(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_kwin_shadow::org_kde_kwin_shadow(struct ::org_kde_kwin_shadow *obj)
        : m_org_kde_kwin_shadow(obj)
    {
    }

    org_kde_kwin_shadow::org_kde_kwin_shadow()
        : m_org_kde_kwin_shadow(nullptr)
    {
    }

    org_kde_kwin_shadow::~org_kde_kwin_shadow()
    {
    }

    void org_kde_kwin_shadow::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_kwin_shadow = static_cast<struct ::org_kde_kwin_shadow *>(wlRegistryBind(registry, id, &org_kde_kwin_shadow_interface, version));
    }

    void org_kde_kwin_shadow::init(struct ::org_kde_kwin_shadow *obj)
    {
        m_org_kde_kwin_shadow = obj;
    }

    org_kde_kwin_shadow *org_kde_kwin_shadow::fromObject(struct ::org_kde_kwin_shadow *object)
    {
        return static_cast<org_kde_kwin_shadow *>(org_kde_kwin_shadow_get_user_data(object));
    }

    bool org_kde_kwin_shadow::isInitialized() const
    {
        return m_org_kde_kwin_shadow != nullptr;
    }

    uint32_t org_kde_kwin_shadow::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_kwin_shadow));
    }

    const struct wl_interface *org_kde_kwin_shadow::interface()
    {
        return &::org_kde_kwin_shadow_interface;
    }

    void org_kde_kwin_shadow::commit()
    {
        ::org_kde_kwin_shadow_commit(
            m_org_kde_kwin_shadow);
    }

    void org_kde_kwin_shadow::attach_left(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_left(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_top_left(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_top_left(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_top(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_top(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_top_right(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_top_right(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_right(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_right(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_bottom_right(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_bottom_right(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_bottom(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_bottom(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::attach_bottom_left(struct ::wl_buffer *buffer)
    {
        ::org_kde_kwin_shadow_attach_bottom_left(
            m_org_kde_kwin_shadow,
            buffer);
    }

    void org_kde_kwin_shadow::set_left_offset(wl_fixed_t offset)
    {
        ::org_kde_kwin_shadow_set_left_offset(
            m_org_kde_kwin_shadow,
            offset);
    }

    void org_kde_kwin_shadow::set_top_offset(wl_fixed_t offset)
    {
        ::org_kde_kwin_shadow_set_top_offset(
            m_org_kde_kwin_shadow,
            offset);
    }

    void org_kde_kwin_shadow::set_right_offset(wl_fixed_t offset)
    {
        ::org_kde_kwin_shadow_set_right_offset(
            m_org_kde_kwin_shadow,
            offset);
    }

    void org_kde_kwin_shadow::set_bottom_offset(wl_fixed_t offset)
    {
        ::org_kde_kwin_shadow_set_bottom_offset(
            m_org_kde_kwin_shadow,
            offset);
    }

    void org_kde_kwin_shadow::destroy()
    {
        ::org_kde_kwin_shadow_destroy(
            m_org_kde_kwin_shadow);
        m_org_kde_kwin_shadow = nullptr;
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
